//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// Photo object
function Photo(title) {
   this.init(title);
}

// matches zoom level of map
Photo.MID_BLOCK          = 1;
Photo.SMALL_INTERSECTION = 2;
Photo.INTERSECTION       = 3;
Photo.BIG_INTERSECTION   = 4;
Photo.MAJOR_INTERSECTION = 5;
Photo.LANDMARK           = 6;
Photo.CAPITOL            = 7;
Photo.MAJOR_LANDMARK     = 8;
Photo.WONDER             = 9;

Photo.key = function(lat, lon) {
   return lat + "x" + lon;
};

Photo.setPrecision = function(value) {
   return Math.round(value * 100000)/100000;
};

Photo.sort = function(photos) {
   var sortedPhotos = {};
   var directions = ["north", "northeast", "east", "southeast", "south", "southwest", "west", "northwest", "unknown"];
   var p;
   for (var d = 0; d < directions.length; d++) {
      for (p in photos) {
         if (photos[p].pov.toLowerCase() == directions[d]) {
            sortedPhotos[p] = photos[p];
         }
      }
   }
   return sortedPhotos;
};



Photo.prototype = {
   version : "00",
   lat : "0.0",
   lon : "0.0",
   pov : "unknown",
   season : "winter",
   tod : "day",
   type : Photo.MID_BLOCK,

   // subclasses supply these
   smallUrl : "",
   mediumUrl : "",
   url : "",

   init : function(title) {
      this.title = escape(title);
   }

};

function FlickrPhoto(staticServer, url, title, id, secret, owner, server) {
   Photo.prototype.init.call(this, title);
   this.init(staticServer, url, id, secret, owner, server);
}

FlickrPhoto.getTags = function(editInfo) {
   editInfo.lat = Photo.setPrecision(editInfo.lat);
   editInfo.lon = Photo.setPrecision(editInfo.lon);
   var twoDecimalLat = PhotoMapUtils.floor(editInfo.lat * 100)/100;
   var twoDecimalLon = PhotoMapUtils.floor(editInfo.lon * 100)/100;
   var oneDecimalLat = PhotoMapUtils.floor(editInfo.lat * 10)/10;
   var oneDecimalLon = PhotoMapUtils.floor(editInfo.lon * 10)/10;
   var shortLat = PhotoMapUtils.floor(editInfo.lat);
   var shortLon = PhotoMapUtils.floor(editInfo.lon);
   var aproxLat = PhotoMapUtils.floor(editInfo.lat/10) * 10;
   var aproxLon = PhotoMapUtils.floor(editInfo.lon/10) * 10;

   // add all they type flags that apply
   var pmTypeTags = "";
   for (var type = editInfo.type; type > 0; type--) {
      pmTypeTags += " pmType:" + type;
   }

   var negativeTags = "";
   if (editInfo.lat < 0) {
      negativeTags += " negativeLat";
   }
   if (editInfo.lon < 0) {
      negativeTags += " negativeLon";
   }

   return   "PhotoMap pmVer01"                  +
            negativeTags                        +
            " aproxLat:"      + aproxLat        +
            " aproxLon:"      + aproxLon        + 
            " shortLat:"      + shortLat        +
            " shortLon:"      + shortLon        + 
            " oneDecimalLon:" + oneDecimalLon   +
            " oneDecimalLat:" + oneDecimalLat   +
            " twoDecimalLon:" + twoDecimalLon   +
            " twoDecimalLat:" + twoDecimalLat   +
            " geo:lat="       + editInfo.lat    +
            " geo:long="      + editInfo.lon    + 
            " "               + editInfo.season +
            " "               + editInfo.tod    + 
            " pmPOV:"         + editInfo.pov    +
            pmTypeTags   + 
            " "               + editInfo.otherInfo;
};

FlickrPhoto.getDescription = function(editInfo) {
   return "Photo uploaded by photomap extension.  \n\n photomap://map?lat=" +
      editInfo.lat + "&long=" + editInfo.lon + "&zoom=" + editInfo.type + "&type=hybrid";
};

FlickrPhoto.getStringValue = function(key, node) {
      return document.evaluate(key, node, null, XPathResult.STRING_TYPE, null).stringValue;
};

FlickrPhoto.fromXML = function(node) {
   var staticServer = FlickrPhoto.getStringValue("staticServer", node);
   var url     = FlickrPhoto.getStringValue("url", node);
   var id      = FlickrPhoto.getStringValue("id", node);
   var secret  = FlickrPhoto.getStringValue("secret", node);
   var owner   = FlickrPhoto.getStringValue("owner", node);
   var server  = FlickrPhoto.getStringValue("server", node);
   var version = FlickrPhoto.getStringValue("version", node);
   var lat     = FlickrPhoto.getStringValue("lat", node);
   var lon     = FlickrPhoto.getStringValue("lon", node);
   var type    = FlickrPhoto.getStringValue("type", node);
   var pov     = FlickrPhoto.getStringValue("pov", node);
   var tod     = FlickrPhoto.getStringValue("tod", node);
   var season  = FlickrPhoto.getStringValue("season", node);
   var other   = FlickrPhoto.getStringValue("other", node);
   var title   = FlickrPhoto.getStringValue("title", node);

   var photo =  new FlickrPhoto(staticServer, url, title, id, secret, owner, server);
   photo.version = version;
   photo.lat = lat;
   photo.lon = lon;
   photo.type = type;
   photo.pov = pov;
   photo.tod = tod;
   photo.season = season;
   photo.other = other;
   return photo;
};

FlickrPhoto.prototype = {
   __proto__ : Photo.prototype,

   init : function(staticServer, imageURL, id, secret, owner, server) {
      this.staticServer = staticServer;
      this.url = imageURL;
      this.id = id;
      this.secret = secret;
      this.owner = owner;
      this.server = server;
      this.regularUrl = this.staticServer  + "/" + server + "/" + id + "_" + secret + ".jpg";
      this.mediumUrl  = this.staticServer  + "/" + server + "/" + id + "_" + secret + "_m.jpg";
      this.smallUrl   = this.staticServer  + "/" + server + "/" + id + "_" + secret + "_s.jpg";
      this.other = "";

      if (staticServer.indexOf("flickr") >= 0) {
         this.provider = "flickr";
      } else if (staticServer.indexOf("23hq") >= 0) {
         this.provider = "23";
      } else {
         this.provider = "unknown";
      }
   },

   // n.b. all keyValue's will be lower case, with no spaces or colons or dashes
   // we need to use the raw value if we care about that (e.g. for lat/lon)
   setValue : function(rawKey) {
      if (rawKey) {
         rawKey = rawKey.toLowerCase();
         if (rawKey.indexOf("pmver") == 0) {
            this.version = rawKey.substr(5);

         // remove these two
         } else if (rawKey.indexOf("lat:") == 0) {
            // we care about minus signs and decimals, and we have "lat:"
            this.lat = rawKey.substr(4);
            this.lat = Photo.setPrecision(this.lat);
         } else if (rawKey.indexOf("lon:") == 0) {
            // might be a minus sign, and we have "lon:"
            this.lon = rawKey.substr(4);
            this.lon = Photo.setPrecision(this.lon);

         } else if (rawKey.indexOf("geo:lat=") == 0) {
            // we care about minus signs and decimals, and we have "geo:lat="
            // only use the first one (it's more likely the newest, which means photomap
            if (this.lat == "0.0") {
               this.lat = rawKey.substr(8);
               this.lat = Photo.setPrecision(this.lat);
            }
         } else if (rawKey.indexOf("geo:long=") == 0) {
            // might be a minus sign, and we have "geo:long="
            this.lon = rawKey.substr(9);
            this.lon = Photo.setPrecision(this.lon);

         } else if (rawKey.indexOf("geo:lon=") == 0) {
            // might be a minus sign, and we have "geo:lon="
            // don't overwrite geo:long= values
            if (this.lon == "0.0") {
               this.lon = rawKey.substr(8);
               this.lon = Photo.setPrecision(this.lon);
            }
         // some bogus ones we don't want to see in other
         } else if (rawKey.indexOf("aprox")      == 0) {
            // do nothing
         } else if (rawKey.indexOf("photomap")   == 0) {
            // do nothing
         } else if (rawKey.indexOf("onedecimal") == 0) {
            // do nothing
         } else if (rawKey.indexOf("twodecimal") == 0) {
            // do nothing
         } else if (rawKey.indexOf("shortl")     == 0) {
            // do nothing
         } else if (rawKey.indexOf("pmtype") == 0) {
            // we get a bunch of these, use the biggest
            var type = rawKey.substr(7);
            if (0 + type > 0 + this.type) {
               this.type = type;
            }
         } else if (rawKey.indexOf("pmpov:") == 0) {
            this.pov = rawKey.substr(6);
         } else if (rawKey.indexOf("day") == 0) {
            this.tod = "day";
         } else if (rawKey.indexOf("night") == 0) {
            this.tod = "night";
         } else if (rawKey.indexOf("summer") == 0) {
            this.season = "summer";
         } else if (rawKey.indexOf("winter") == 0) {
            this.season = "winter";
         } else if (rawKey.indexOf("spring") == 0) {
            this.season = "spring";
         } else if (rawKey.indexOf("autumn") == 0) {
            this.season = "autumn";
         } else {
            this.other += escape(rawKey) + " ";
         }
      }
   },

   key : function() {
      return Photo.key(this.lat, this.lon);
   },

   toXML : function() {
      var xmlString;
      xmlString = "<photo>\n";
      xmlString += this.getTag("title"        );
      xmlString += this.getTag("id"           );
      xmlString += this.getTag("secret"       );
      xmlString += this.getTag("owner"        );
      xmlString += this.getTag("server"       );
      xmlString += this.getTag("version"      );
      xmlString += this.getTag("lat"          );
      xmlString += this.getTag("lon"          );
      xmlString += this.getTag("type"         );
      xmlString += this.getTag("pov"          );
      xmlString += this.getTag("tod"          );
      xmlString += this.getTag("season"       );
      xmlString += this.getTag("other"        );
      xmlString += this.getTag("staticServer" );
      xmlString += this.getTag("url");
      xmlString += "</photo>\n";
      return xmlString;
   },

   getTag : function(id) {
      return "<" + id + ">" + this[id] + "</" + id + ">\n";
   }
};

