//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


function PhotoMapController(gui, location) {
   this.init(gui, location);
}

PhotoMapController.prototype = {
   preferences : PhotoMapPreferences.instance(),
   isBigPhotos : false,

   // Initialisation and termination functions
   init : function(gui, location) {
      this.photoMode = "view";
      this.gui = gui;
      if (this.preferences.getResizableMainPhoto()) {
         this.gui.setMainPhotoResizable();
      }
      if (this.preferences.getBigPhotos()) {
         this.isBigPhotos = true;
         this.gui.setBigPhotos();
      }
      this.photo = new PhotoMapPhotoAgent(this);
      this.map   = new PhotoMapMapAgent(this);
      this.geocoder = new PhotoMapGeocoderAgent(this);
      if (location && location.lat && location.lon) {
         this.startingPoint = location;
      } else {
         this.startingPoint = this.preferences.getHome();
      }
      this.map.createMap(this.gui.getMapIFrame());
   },
   
   shutdown : function() {
      this.photo.shutdown();
      this.map.shutdown();
   },

   launchOptions : function() {
      var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
      var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
      var topWindow = windowManagerInterface.getMostRecentWindow('global:PhotoMapOptions');
      if (topWindow) {
         topWindow.focus();
      } else {
         window.openDialog("chrome://photomap/content/PhotoMapOptions.xul",
                           "PhotoMapOptions",
                           "chrome,extrachrome,menubar,toolbar,dialog=yes,modal,titlebar");
      }
   },

   goHome : function() {
      var point = this.preferences.getHome();
      this.map.setLocation(point, this.preferences.getHomeZoom());
   },

   setHome : function() {
      var point = this.map.getLocation();
      var zoom = this.map.getZoom();
      this.preferences.setHome(point);
      this.preferences.setHomeZoom(zoom);
      this.updateTransactionTotals();
   },

   mapInit : function() {
      this.goHome();
      this.setLocation(this.startingPoint);
      delete this.startingPoint;
   },

   mapMoved : function(force) {
      // this is called by the map object whenever we move or zoom
      var loc = this.map.getLocation();
      var bounds = this.map.getBounds();
      var zoom = this.map.getZoom();
      this.gui.setMapInfo(loc, bounds, zoom);

      this.updateMarkers();

      if (this.isView()) {
         // let the photo controller know we have a new target
         this.photo.locationChanged(loc, bounds, zoom, force);
      }
      this.updateTransactionTotals();
   },

   refresh : function() {
      this.mapMoved(true);
   },

   updateMarkers : function() {
      this.updateCurrentMarkers();
      if (this.isEdit()) {
         this.map.updateCenterMarker();
      }   
   },

   updateCurrentMarkers : function() {
      this.map.updateCurrentMarkers();
   },

   markerUpdate : function(key) {
      this.updateMarkersByKey(key);
      // change this to update debug info or something
      // and put all of that stuff on a div we can hide/show
      this.updateTransactionTotals();
   },


   updateMarkersByKey : function(key) {
      var mapThings = this.photo.getByKey(key);
      // we really only need to call this once, because the key is a map location, and we don't want
      // more than one marker in the same place
      for (thingKey in mapThings) {
         // map should deal with muliple calls to add marker,
         this.map.addMarker(mapThings[thingKey]);
         break;
      }
   },

   markerSelected : function(lat, lon) {
      // find the image
      var imageSet = this.photo.getByLatLon(lat, lon);
      if (imageSet) {
         imageSet = Photo.sort(imageSet);
         this.gui.setPhotos(imageSet);
      }
   },

   isEdit : function() {
      return this.photoMode == "edit";
   },

   isView : function() {
      return !this.isEdit();
   },

   setPhotoViewMode : function() {
      this.photoMode = "view";
      this.map.clearMarkers();
      this.mapMoved();
   },

   setPhotoEditMode : function() {
      this.photoMode = "edit";
      this.photo.stop();
      this.map.clearMarkers();
      this.mapMoved();

      this.gui.setNeedsAuthentication(this.photo.needsAuthentication());
   },

   setLocation : function(loc) {
      this.map.setLocation(loc);
      if (loc.zoom) {
         this.map.setLocation(loc, loc.zoom);
      }
      if (loc.type) {
         this.map.setMapType(loc.type);
      }
   },

   getLocation : function() {
      var loc = this.map.getLocation();
      loc.zoom = this.map.getZoom();
      return loc;
   },

   gotoPhoto : function(photo) {
      if (photo) {
         var loc = {lat : photo.lat, lon : photo.lon};
         this.setLocation(loc);
      }
   },

   getEditInfo : function() {
      var loc = this.map.getLocation();
      var lat = loc.lat;
      var lon = loc.lon;
      var type = this.gui.getEditType();
      var zoom = this.map.getZoom();
      var season = this.gui.getEditSeason();
      var tod = this.gui.getEditTOD();
      var pov = this.gui.getEditPOV();
      var mapType = this.map.getType();
      var other = " " + this.gui.getEditOtherTags() + " ";
      var defaultTags = this.preferences.getDefaultTags();
      if (defaultTags) {
         if (other.indexOf(" " + defaultTags + " ") == -1) { 
            other = defaultTags + other;
         }
      }

      return new EditInfo(lat, lon, type, zoom, season, tod, pov, mapType, other);
   },

   updateTags : function() {
      // move this to photo agent -> object
      var tags = this.photo.getTags(this.getEditInfo());
      this.gui.setEditOutputTags(tags);
   },

   photoAuthenticate : function() {
      this.photo.authenticate();
   },

   photoDoneAuthenticate : function() {
      this.photo.doneAuthenticate();
   },

   openWindow : function(url) {
      this.gui.openWindow(url);
   },

   getNewAccountUrl : function() {
      return this.photo.getNewAccountUrl();
   },

   authenticationSuccess : function() {
      this.gui.setAuthenticated();
   },

   getPhotoUserName : function() {
      return this.photo.getUserName();
   },

   getPhotoTotalPictures : function() {
      return this.photo.getTotalPictures();
   },

   photoLogout : function() {
      this.photo.logout();
   },

   loadFile : function() {
      var photoFile = PhotoMapUtils.getImageFile("Select a file:");
      if (photoFile) {
         this.photoFile = photoFile;
         this.gui.setPhotoFile(photoFile.target);
      }
   },

   upload : function() {
      if (this.photoFile) {
         var editInfo = this.getEditInfo();
         editInfo.file = this.photoFile;
         this.photo.upload(editInfo, new Callback(this, this.doneUpload));
      }
   },

   update : function() {
      if (this.editPhoto) {
         var editInfo = this.getEditInfo();
         editInfo.photo = this.editPhoto;
         this.photo.update(editInfo, new Callback(this, this.doneUpdate));
      }
   },

   doneUpload : function(wasError) {
      this.gui.doneUpload(wasError);
      this.photoFile = null;
   },

   doneUpdate : function(wasError) {
      this.gui.doneUpdate(wasError);
      this.editPhoto = null;
   },

   launchHelp : function() {
      this.gui.openNewTab("http://photomap.mozdev.org/help.html");
   },

   copyTags : function(tags) {
      PhotoMapUtils.copyToClipboard(tags);
   },

   // this is evil
   updateTransactionTotals : function() {
      var transactionsPerSecond = BaseFlickrPhotoTransaction.transactionsPerSecond("flickr") + " " + BaseFlickrPhotoTransaction.transactionsPerSecond("23") + " txn/sec ";
      var outstandingTransactions = "[" + BaseFlickrPhotoTransaction.outstandingTransactions("flickr") + "][" + BaseFlickrPhotoTransaction.outstandingTransactions("23") + "]";
      this.gui.updateTransactionTotal(transactionsPerSecond + outstandingTransactions);
   },

   copyLinkText : function() {
      var editInfo = this.getEditInfo();
      var link = editInfo.uri();
      PhotoMapUtils.copyToClipboard(link);
   },

   geocode : function(location) {
      this.geocoder.geocode(new Callback(this, this.doneGeocode), location);
   },

   doneGeocode : function(location, wasError) {
      if (wasError) {
         window.alert("Couldn't find address");
      } else {
         this.setLocation(location);
      }
   },

   getPhotoServiceName : function() {
      // move this into this.photo?
      return this.preferences.getPhotoServiceType();
   },

   photoLocationDropped : function(location) {
      // we'll get called back when it's done
      try {
         this.photo.getPhotoInfo(location);
      } catch (e) {
         this.gui.problemWithPhotoDrop(e);
      }
   },

   fileDropped : function(file) {
      if (file) {
         this.editPhoto = null;
         this.gui.setEditMode();
         this.photoFile = file;
         this.gui.setPhotoFile(file.target);
      }
   },

   // called after we look up information from a dropped photo
   photoLocationInfo : function(wasError, photo) {
      if (!wasError) {
         this.editPhoto = photo;
         this.photoFile = null;
         this.gui.setEditMode();
         if ((photo.lat != "0.0") && (photo.lon != "0.0")) {
            this.setLocation({lat: photo.lat, lon : photo.lon, zoom : photo.type, type : "hybrid"});
         }
         this.gui.photoUpdate(photo);
      } else {
         this.gui.problemWithPhotoDrop(wasError);
      }
   }
};

function EditInfo(lat, lon, type, zoom, season, tod, pov, maptype, other) {
   this.init(lat, lon, type, zoom, season, tod, pov, maptype, other);
}

EditInfo.prototype = {
   init : function(lat, lon, type, zoom, season, tod, pov, maptype, other) {
      this.lat = lat;
      this.lon = lon;
      this.type = type;
      this.zoom = zoom;
      this.season = season;
      this.tod = tod;
      this.pov = pov;
      this.maptype = maptype;
      this.otherInfo = other;
   },

   uri : function() {
      return "photomap://map?lat=" + this.lat +"&long=" + this.lon + "&zoom=" + this.zoom + "&type=" + this.maptype;
   }
};

