//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// the wrapper for our specific map implementation
function PhotoMapMapAgent(controller) {
   this.init(controller);
}

PhotoMapMapAgent.prototype = {
   init : function(controller) {
      this.addToListener();
      this.controller = controller;
      this.preferences = controller.preferences;
      this.type = this.preferences.getMapType();
      this.markers = {};
      this.iMap = this.createMapImplementation();
      this.setupMap();
   },

   shutdown : function() {
      this.removeFromListener();
   },

   // not really happy with this factory, but it seems
   // like a waste to make it more complicated, as all the
   // implementations need to be included in the .xul file anyway
   createMapImplementation : function() {
      var iMap;
      if (this.type == "google") { 
         iMap = new PhotoMapGMap();
      } else if (this.type == "googlev2") { 
         iMap = new PhotoMapGMapV2();
      } else if (this.type == "yahoo") {
         iMap = new PhotoMapYMap();
      } else {
         window.alert("Map type not implemented [" + this.type + "]");
      }
      return iMap;
   },

   setupMap : function() {
      if (this.iMap) {
         PhotoMapUtils.createMapSource(this.iMap.getURL(this.preferences), this.iMap.getContent(this.preferences));
      }
   },
   
   createMap : function(iframe) {
      this.iframe = iframe;
      if (this.iMap) {
         this.iMap.create(iframe, this.preferences);
         this.asyncCallInit();
      }
   },

   asyncCallInit : function() {
      var mmap = this;
      PhotoMapUtils.callAsync(this, this.mapInit, 2000, [mmap]);
   },

   mapInit : function() {
      if (this.iMap) {
         try {
            this.iMap.ensureInit(this.iframe);
            PhotoMapUtils.callAsync(this.controller, this.controller.mapInit, 500, []);
            // clean up cache
            PhotoMapUtils.cleanMapSource(this.iMap.getURL(this.preferences));
         } catch (e) {
            PhotoMapUtils.log("Caught an exception in mapInit: " + e);
            this.asyncCallInit();
         }
      }
   },

   setLocation : function(loc, zoom) {
      if (loc && loc.lon && loc.lat) {
         if (this.iMap) {
            this.iMap.ensureInit();
            if (!zoom) {
               zoom = this.iMap.getZoom();
            }
            this.iMap.setLocation(loc.lat, loc.lon, zoom);
         } else {
            this.init(this.controller);
            this.createMap(this.iframe);
            var mmap = this;
            window.setTimeout(function() {PhotoMapMapAgent.prototype.setLocation.call(mmap, lat, lon, zoom);}, 2000);
         }
      } else {
         if (!loc) {
            loc = "null";
         }
         PhotoMapUtils.log("There was a problem with the location: [" + loc.toString() + "]");
      }
   },

   setZoom : function(zoom) {
      if (this.iMap) {
         this.iMap.ensureInit();
         this.iMap.setLocation(loc.lat, loc.lon, zoom);
      } else {
         this.init(this.controller);
         this.createMap(this.iframe);
         var mmap = this;
         window.setTimeout(function() {PhotoMapMapAgent.prototype.setLocation.call(mmap, lat, lon, zoom);}, 2000);
      }
   },

   setMapType : function(type) {
      this.iMap.setType(type);
   },

   getType : function() {
      return this.iMap.getType();
   },

   getLocation : function() {
      if (this.iMap) {
         this.iMap.ensureInit();
         return this.iMap.getLocation();
      } else {
         return undefined;
      }
   },

   getBounds : function() {
      if (this.iMap) {
         this.iMap.ensureInit();
         return this.iMap.getBounds();
      } else {
         return undefined;
      }
   },

   
   // the map implementations are responsable for
   // normalizing the zoom
   getZoom : function() {
      var zoom = 15;
      if (this.iMap) {
         zoom = this.iMap.getZoom();
      }
      return zoom;
   },

   mapMoved : function() {
      this.controller.mapMoved();
   },

   updateCurrentMarkers : function() {
      this.iMap.ensureInit();

      for (var currentMarker in this.markers) {
         this.addMarker(this.markers[currentMarker]);
      }
   },

   markerSelected : function(lat, lon) {
      this.controller.markerSelected(lat, lon);
   },

   markerUnSelected : function() {
      // this.controller.markerUnSelected(photo);
   },

   addMarker : function(photo) {
      this.iMap.ensureInit();
      var added = false;
      if (photo.type >= this.getZoom()) {
         if (this.inBounds(photo)) {
            this.iMap.addMarker(photo);
            this.markers[photo.key()] = photo;
            added = true;
         }
      }
      if (!added) {
         this.iMap.removeMarker(photo);
         if (this.markers[photo]) {
            delete this.markers[photo.key()];
         }
      }
   },

   selectMarker : function(photo) {
      this.iMap.selectMarker(photo);
   },

   // should we outset/inset this?
   inBounds : function(photo) {
      var bounds = this.getBounds();
      if (bounds && photo) {
         if (photo.lat < bounds.maxY && 
             photo.lat > bounds.minY && 
             photo.lon < bounds.maxX &&
             photo.lon > bounds.minX) {
            return true;
         }
      }
      return false;
   },

   clearMarkers : function() {
      this.iMap.clearMarkers();
      this.markers = {};
   },

   updateCenterMarker : function() {
      this.clearMarkers();
      this.iMap.addCenterMarker();
   },


   // observer interface stuff to listed to map implementation
   QueryInterface: function(iid) {
    if (!iid.equals(Components.interfaces.nsISupports) &&
        !iid.equals(Components.interfaces.nsIObserver)) {
          throw Components.results.NS_ERROR_NO_INTERFACE;
      }
      return this;
    },

   addToListener: function() {
      // Register new request and response listener
      var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      observerService.addObserver(this, "photoMap-mapMoved", false);
      observerService.addObserver(this, "photoMap-markerSelected", false);
   },
   
   removeFromListener: function() {
      // Unregistering listener
      var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      observerService.removeObserver(this, "photoMap-mapMoved");
      observerService.removeObserver(this, "photoMap-markerSelected");
   },

   observe: function(aSubject, aTopic, aData) {
      if (aTopic == 'photoMap-mapMoved') {
         this.mapMoved();
      } else if (aTopic == "photoMap-markerSelected") {
         // data is latxlon
         this.markerSelected(aData.substring(0, aData.indexOf("x")), aData.substring(aData.indexOf("x") + 1));
      }
   }
};
