//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// TimedQueue
function TimedQueue(msDelay, batchSize, callback, testCallback) {
   this.init(msDelay, batchSize, callback, testCallback);
}

TimedQueue.prototype = {
   init : function(msDelay, batchSize, callback, testCallback) {
      this.delay = msDelay;
      this.batchSize = batchSize;
      this.callback = callback;
      this.testCallback = testCallback;
      this.size = 0;
      this.head = null;
      this.tail = null;
      // no timer until we get some info
      this.timerRunning = false;
   },

   add : function(info) {
      var obj = {value : info};
      var oldTail = this.tail;
      if (oldTail) {
         oldTail.prev = obj;
         obj.next = oldTail;
      }
      this.tail = obj;

      if (!this.head) {
         this.head = obj;
      }
      this.size++;
      this.startTimer();
   },

   remove : function() {
      var obj = null;
      var value = null;
      if (this.head) {
         obj = this.head;
         this.head = this.head.prev;
         if (!this.head) {
            this.tail = null;
         } else {
            this.head.next = null;
         }
         this.size--;
      }
      if (obj) {
         value = obj.value;
      }
      return value;
   },

   clear : function() { 
      this.size = 0;
      this.head = null;
      this.tail = null;
   },

   queueSize : function() {
      return this.size;
   },

   startTimer : function() {
      if (!this.timerRunning) {
         this.timerRunning = true;
         PhotoMapUtils.callAsync(this, this.execute, this.delay, []);
      }
   },

   execute : function() {
      this.timerRunning = false;
      if (this.queueSize()) {
         // if the queue is non-empty, trigger another event
         PhotoMapUtils.log("TimedQueue.execute: " + this.size);
         for (var batchCount = 0; batchCount < this.batchSize; batchCount++) {
            // if we have a test, run it and short circuit if it fails
            if ((this.testCallback && this.testCallback.execute(this)) || !this.testCallback) {
               var obj = this.remove();
               if (obj) {
                  this.callback.execute(obj);
               } else {
                  break;
               }
            } else {
               PhotoMapUtils.log("TimedQueue.execute - break - test callback failed");
               break;
            }
         }
         if (this.queueSize()) {
            this.startTimer();
         }
      }
   }
};

